# src/const/column_names.py

# 共通
TIMESTAMP = "ts"

# 加速度計
ACC_X = "acc_x"
ACC_Y = "acc_y"
ACC_Z = "acc_z"

# ジャイロスコープ
GYRO_X = "gyro_x"
GYRO_Y = "gyro_y"
GYRO_Z = "gyro_z"

# 角度 ジャイロを積分したもの
ANGLE = "angle_x"

# 気圧計
PRESSURE = "pressure"

# BLE
BLE_ADDRESS = "bdaddress"
RSSI = "rssi"

# 位置
POS_X = "x"
POS_Y = "y"
POS_Z = "z"

# 四元数
QUATERNION_0 = "q0"
QUATERNION_1 = "q1"
QUATERNION_2 = "q2"
QUATERNION_3 = "q3"

# フロア名
FLOOR_NAME = "floor_name"

# 座標
COORDINATE_X = "x"
COORDINATE_Y = "y"


# step
STEP_LENGTH = "step_length"
